/** 
 * @file gl860-dev-0503.c
 * @author Olivier LORIN, tiré du pilote Syntek par Nicolas VIVIEN
 * @date 2008-05-05
 * @version v0.9.0
 *
 * @brief Driver for GL860 USB video camera
 *
 * @note Copyright (C) Olivier LORIN
 *
 * @par Licences
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <linux/module.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/version.h>
#include <linux/errno.h>
#include <linux/slab.h>
#include <linux/kref.h>

#include <linux/usb.h>
#include <media/v4l2-common.h>

#include "gl860.h"
#include "gl860-dev.h"

#define ctrl_in fct_RTx
#define ctrl_out fct_RTx

// Code issu des logs de Ice/Soro2005 (A), Fret_saw/Hulkie(B) et Tricid (C)
// avec l'aide de Kytrix/Blazercist
// Capteur : MI2020

static int exlum=-1;
static int exmirh=-1;
static int exmirv=-1;
static int exnet=-1;
static int excntr=-1;
static int excontrej=-1;
static int exfreq=-1;
static int exgam=-1;

static void commun(struct usb_gl860 *dev);


char nettete[]={ 0x6c, 0x00, 0x08 };

char lum_a[]={ 0x8c, 0xa2, 0x06 };
char lum_b[]={ 0x90, 0x00, 0x00 };
char lum_c[]={ 0x8c, 0xa1, 0x02 };
char lum_d[]={ 0x90, 0x00, 0x0f };
char lum_e[]={ 0x8c, 0xa1, 0x03 };
char lum_f[]={ 0x90, 0x00, 0x05 };

char rien1[]={ 0x90, 0x00, 0x06 };
char rien2[]={ 0x8c, 0xa1, 0x02 };
char rien3[]={ 0x90, 0x00, 0x1f };

char mir_a[]={ 0x8c, 0x27, 0x19 };
char mir_b[]={ 0x90, 0x04, 0x6c };
char mir_c[]={ 0x8c, 0x27, 0x3b };
char mir_d[]={ 0x90, 0x00, 0x24 };
char mir_e[]={ 0x8c, 0xa1, 0x03 };
char mir_f[]={ 0x90, 0x00, 0x06 };

char freq_a[]={ 0x8c, 0xa4, 0x04 };
char freq_b[]={ 0x90, 0x00, 0x80 };

char cjn_a[]={ 0x8c, 0xa7, 0x00 };
char cjn_b[]={ 0x90, 0x00, 0x00 };
char cjn_c[]={ 0x8c, 0xa7, 0x00 };
char cjn_d[]={ 0x90, 0x00, 0x00 };
char cjn_e[]={ 0x8c, 0xa1, 0x03 };
char cjn_f[]={ 0x90, 0x00, 0x05 };





// Partie commune à l'initialisation et à la mise en marche
static void commun(struct usb_gl860 *dev)
{
//#ifdef _0503A_
if (_0503A_)
{
	ctrl_out(dev, 0x40, 1, 0x0000, 0x0000, 0, NULL); msleep(35);
	ctrl_out(dev, 0x40, 1, 0x006a, 0x0007, 0, NULL); msleep(35);
	ctrl_out(dev, 0x40, 1, 0x0063, 0x0006, 0, NULL); msleep(35);
	ctrl_out(dev, 0x40, 1, 0x006a, 0x000d, 0, NULL); msleep(35);
	ctrl_out(dev, 0x40, 1, 0x0000, 0x00c0, 0, NULL);
	ctrl_out(dev, 0x40, 1, 0x0010, 0x0010, 0, NULL);
	ctrl_out(dev, 0x40, 1, 0x0003, 0x00c1, 0, NULL);
	ctrl_out(dev, 0x40, 1, 0x0042, 0x00c2, 0, NULL);
	ctrl_out(dev, 0x40, 1, 0x0004, 0x00d8, 0, NULL);
	ctrl_out(dev, 0x40, 1, 0x0000, 0x0058, 0, NULL);
	ctrl_out(dev, 0x40, 1, 0x0002, 0x0004, 0, NULL);
	ctrl_out(dev, 0x40, 1, 0x0041, 0x0000, 0, NULL);
} else {
//#else
//	#ifdef _0503C_
	if (_0503C_)
	{
  	ctrl_out(dev, 0x40, 1, 0x0008, 0x0004, 0, NULL); msleep(35);
	}
//	#else
	else
	{
   ctrl_out(dev, 0x40, 1, 0x0002, 0x0004, 0, NULL); msleep(35);
	}
//	#endif
   ctrl_out(dev, 0x40, 1, 0x006a, 0x0007, 0, NULL); msleep(35);
   ctrl_out(dev, 0x40, 1, 0x00ef, 0x0006, 0, NULL); msleep(35);
   ctrl_out(dev, 0x40, 1, 0x006a, 0x000d, 0, NULL); msleep(35);
   ctrl_out(dev, 0x40, 1, 0x0000, 0x00c0, 0, NULL);
   ctrl_out(dev, 0x40, 1, 0x0010, 0x0010, 0, NULL);
   ctrl_out(dev, 0x40, 1, 0x0003, 0x00c1, 0, NULL);
   ctrl_out(dev, 0x40, 1, 0x0042, 0x00c2, 0, NULL);
   ctrl_out(dev, 0x40, 1, 0x0004, 0x00d8, 0, NULL);
   ctrl_out(dev, 0x40, 1, 0x0000, 0x0058, 0, NULL);
   ctrl_out(dev, 0x40, 1, 0x0041, 0x0000, 0, NULL);
//#endif
}
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x86\x25\x01");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x86\x25\x00");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0030,  3, "\x1a\x0a\xcc");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0032,  3, "\x02\x00\x08");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\xf4\x03\x1d"); msleep(12);
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\x1e\x8f\x09");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\x1c\x01\x28");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\x1e\x8f\x09");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\x1e\x8f\x09");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0032,  3, "\x14\x06\xe6");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x22\x23");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa2\x0f");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x0d");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa2\x10");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x0b");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa2\x11");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x07");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\xf4\x03\x1d");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\xa2\x00\xe2");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xab\x05");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x01");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0032,  3, "\x6e\x00\x86");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0032,  3, "\x70\x0f\xaa");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0032,  3, "\x72\x0f\xe4");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa3\x4a");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x5a");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa3\x4b");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\xa6");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa3\x61");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\xc8");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa3\x62");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\xe1");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xce\x01\xa8");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xd0\x66\x33");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xd2\x31\x9a");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xd4\x94\x63");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xd6\x4b\x25");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xd8\x26\x70");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xda\x72\x4c");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xdc\xff\x04");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xde\x01\x5b");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xe6\x01\x13");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xee\x0b\xf0");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xf6\x0b\xa4");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x00\xf6\xe7");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x08\x0d\xfd");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x10\x25\x63");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x18\x35\x6c");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x20\x42\x7e");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x28\x19\x44");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x30\x39\xd4");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x38\xf5\xa8");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x4c\x07\x90");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x44\x07\xb8");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x5c\x06\x88");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x54\x07\xff");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xe0\x01\x52");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xe8\x00\xcc");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xf0\x0d\x83");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xf8\x0c\xb3");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x02\xfe\xba");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x0a\x04\xe0");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x12\x1c\x63");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x1a\x2b\x5a");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x22\x32\x5e");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x2a\x0d\x28");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x32\x2c\x02");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x3a\xf4\xfa");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x4e\x07\xef");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x46\x07\x88");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x5e\x07\xc1");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x56\x04\x64");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xe4\x01\x15");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xec\x00\x82");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xf4\x0c\xce");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xfc\x0c\xba");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x06\x1f\x02");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x0e\x02\xe3");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x16\x1a\x50");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x1e\x24\x39");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x26\x23\x4c");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x2e\xf9\x1b");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x36\x23\x19");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x3e\x12\x08");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x52\x07\x22");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x4a\x03\xd3");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x62\x06\x54");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x5a\x04\x5d");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xe2\x01\x04");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xea\x00\xa0");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xf2\x0c\xbc");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xfa\x0c\x5b");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x04\x17\xf2");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x0c\x02\x08");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x14\x28\x43");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x1c\x28\x62");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x24\x2b\x60");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x2c\x07\x33");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x34\x1f\xb0");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x3c\xed\xcd");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x50\x00\x06");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x48\x07\xff");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x60\x05\x89");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x58\x07\xff");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x40\x00\xa0");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x42\x00\x00");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0032,  3, "\x10\x01\xfc");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x18");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x3c");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x78\x00\x00");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\xb8\x1f\x20");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa2\x06");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x10");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa2\x07");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x08");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa2\x42");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x0b");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa2\x4a");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x8c");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\xba\xfa\x08");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa2\x02");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x22");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa2\x03");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\xbb");
//#if defined (_0503A_) || defined (_0503C_)
if (_0503A_ || _0503C_)
{
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x22\x2e");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\xa0");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa4\x08");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x1f");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa4\x09");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x21");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa4\x0a");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x25");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa4\x0b");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x27");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x24\x11");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\xa0");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x24\x13");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\xc0");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x24\x15");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\xa0");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x24\x17");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\xc0");
	}
//#endif
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa4\x04");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x80");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa7\x9d");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa7\x9e");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa2\x0c");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x17");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa2\x15");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x04");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa2\x14");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x20");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x17"); msleep(53);
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x21\x11");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x1b");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x02\x4f");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x25");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x06\x0f");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x39");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x21\x11");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x3d");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x01\x20");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x47");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x09\x4c");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x03");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x02\x84");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x05");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x01\xe2");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x07");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x06\x40");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x09");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x04\xb0");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x0d");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x0f");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x11");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x04\xbd");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x13");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x06\x4d");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x15");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x17");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x21\x11");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x19");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x04\x6c");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x1b");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x02\x4f");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x1d");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x01\x02");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x1f");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x02\x79");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x21");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x01\x55");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x23");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x02\x85");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x25");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x06\x0f");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x27");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x20\x20");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x29");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x20\x20");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x2b");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x10\x20");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x2d");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x20\x07");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x2f");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x04");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x31");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x04");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x33");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x04\xbb");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x35");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x06\x4b");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x37");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x39");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x21\x11");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x3b");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x24");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x3d");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x01\x20");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x41");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x01\x69");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x45");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x04\xed");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x47");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x09\x4c");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x51");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x53");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x03\x20");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x55");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x57");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x02\x58");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x5f");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x61");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x06\x40");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x63");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x65");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x04\xb0");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x22\x2e");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\xa1");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa4\x08");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x1f");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa4\x09");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x21");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa4\x0a");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x25");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa4\x0b");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x27");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x24\x11");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\xa1");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x24\x13");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\xc1");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x24\x15");
//#if defined (_0503A_) || defined (_0503C_)
if (_0503A_ || _0503C_)
{
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x78"); // diff fret
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x24\x17");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x90"); // diff fret
}
else
//#else
{
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x6a");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x24\x17");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x80");
}
//#endif
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x05");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06");
}







//=============================================================================
//
// STK-M811 API / STK-A311 API
//
//=============================================================================

/** 
 * @param dev Device structure
 * 
 * @returns 0 if all is OK
 *
 * @brief This function permits to initialize the device.
 *
 * This function must be called at first. It's the start of the
 * initialization process. After this process, the device is
 * completly initalized and it's ready.
 *
 * This function is written from the USB log.
 */
int dev_0503his_initialize_device(struct usb_gl860 *dev)
{
//	unsigned char sonde;

	STK_DEBUG("dev_0503his_initialize_device [ini]\n");

//	ctrl_in(dev, 0xc0, 2, 0x0000, 0x0004, 1, &sonde);
//	ctrl_in(dev, 0xc0, 2, 0x0000, 0x0004, 1, &sonde);

	ctrl_out(dev, 0x40, 1, 0x0000, 0x0000, 0, NULL); msleep(53);
	ctrl_out(dev, 0x40, 1, 0x0010, 0x0010, 0, NULL); msleep(53);
	ctrl_out(dev, 0x40, 1, 0x0008, 0x00c0, 0, NULL); msleep(53);
	ctrl_out(dev, 0x40, 1, 0x0001, 0x00c1, 0, NULL); msleep(53);
	ctrl_out(dev, 0x40, 1, 0x0001, 0x00c2, 0, NULL); msleep(53);
	ctrl_out(dev, 0x40, 1, 0x0020, 0x0006, 0, NULL); msleep(53);
	ctrl_out(dev, 0x40, 1, 0x006a, 0x000d, 0, NULL); msleep(53);
/*
	ctrl_out(dev, 0x40, 1, 0x0040, 0x0000, 0, NULL); msleep(5);
	ctrl_out(dev, 0x40, 1, 0x0063, 0x0006, 0, NULL); msleep(5);
	ctrl_out(dev, 0x40, 1, 0x7a00, 0x8030, 0, NULL);
	ctrl_in(dev, 0xc0, 2, 0x7a00, 0x8030, 1, &sonde);
	STK_INFO("probe 1=%2x\n",sonde);

	ctrl_in(dev, 0xc0,  2, 0x0000, 0x0000,  1, &sonde);
	STK_INFO("probe 2=%2x\n",sonde);
	ctrl_in(dev, 0xc0,  2, 0x0000, 0x0000,  1, &sonde);
	STK_INFO("probe 2=%2x\n",sonde);
*/
	commun(dev);

	STK_DEBUG("dev_0503his_initialize_device [fini]\n");
	return 0;
}


/** 
 * @param dev Device structure
 * @param step The step of configuration [0-11]
 * 
 * @returns 0 if all is OK
 *
 * @brief This function permits to configure the device.
 *
 * The configuration of device is composed of 11 steps.
 * This function is called by the initialization process.
 *
 * We don't know the meaning of these steps ! We only replay the USB log.
 *
 * The steps 0 to 9 are called during the initialization.
 * Then, the driver choose the last step :
 *   10 : for a resolution from 80x60 to 640x480 
 *   11 : for a resolution from 800x600 to 1280x1024
 */
int dev_0503his_configure_device(struct usb_gl860 *dev, int step)
{
	return 0;
}


/** 
 * @param dev Device structure
 * 
 * @returns 0 if all is OK
 *
 * @brief Wake-up the camera.
 *
 * This function permits to wake-up the device.
 */
int dev_0503his_camera_asleep(struct usb_gl860 *dev)
{
	return 0;
}


/** 
 * @param dev Device structure
 * 
 * @returns 0 if all is OK
 *
 * @brief This function initializes the device for the stream.
 *
 * It's the start. This function has to be called at first, before
 * enabling the video stream.
 */
int dev_0503his_init_camera(struct usb_gl860 *dev)
{
	return 0;
}


/** 
 * @param dev Device structure
 * 
 * @returns 0 if all is OK
 *
 * @brief This function permits to set default sensor settings.
 *
 * We set some registers in using a I2C bus.
 * WARNING, the sensor settings can be different following the situation.
 */
int dev_0503his_sensor_settings(struct usb_gl860 *dev)
{
	return 0;
}


/** 
 * @param dev Device structure
 * 
 * @returns 0 if all is OK
 *
 * @brief This function permits to modify the settings of the camera.
 *
 * This functions permits to modify the settings :
 *   - brightness
 *   - contrast
 *   - white balance
 *   - ...
 */
int dev_0503his_camera_settings(struct usb_gl860 *dev)
{
int lum  = (dev->vsettings.brightness/256*129)/256;
int mirh = (dev->vsettings.miroirh > 0);
int mirv = (dev->vsettings.miroirv > 0);
int net	= (dev->vsettings.net/256*41)/256;
int cntr	= (dev->vsettings.contrast/256*4)/256;
int contrej	= (dev->vsettings.contrej/256*65)/256;
int freq	= dev->vsettings.freq50;
int gam	= (dev->vsettings.whiteness/256*3)/256;



	if (freq!=exfreq)
	{
		// Fréquence du courant
		freq_b[2]=(freq)?0xc0:0x80;
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)freq_a);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)freq_b);
		exfreq=freq;
	}

	if (mirh!=exmirh || mirv!=exmirv)
	{
		// Miroirs
		if (dev->resolution==WCAM_1280x960 || dev->resolution==WCAM_1600x1200)
		{
			mir_b[2]=0x6c+(mirv)+2*(mirh);
			mir_d[2]=0x24+(mirv)+2*(mirh);
		}
		else
		{
			mir_b[2]=0x6c+(1-mirv)+2*(1-mirh);
			mir_d[2]=0x24+(1-mirv)+2*(1-mirh);
		}

		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)mir_a);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)mir_b);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)mir_c);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)mir_d);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)mir_e);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)mir_f);

		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)rien1);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)rien1);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)rien1);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)rien1);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)rien1);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)rien1);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)rien2);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)rien3);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)rien2);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)rien3);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)rien2);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)rien3);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)rien2);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)rien3);
		exmirh=mirh;
		exmirv=mirv;
	}

	if (contrej!=excontrej)
	{
		// Contre-jour
		cjn_a[2]=0x9d; cjn_c[2]=cjn_a[2]+1;
		cjn_d[2]=cjn_b[2]=contrej;
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_a);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_b);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_c);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_d);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_e);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_f);
		excontrej=contrej;
	}
	if (cntr!=excntr)
	{
		// Contraste
		cjn_a[2]=0x6d; cjn_c[2]=cjn_a[2]+1;
		cjn_d[2]=cjn_b[2]=0x12+16*cntr;
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_a);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_b);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_c);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_d);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_e);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_f);
		excntr=cntr;
	}

	if (gam!=exgam)
	{
		// Gamma
		cjn_a[2]=0x6d; cjn_c[2]=cjn_a[2]+1;
		cjn_d[2]=cjn_b[2]=0x40+gam;
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_a);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_b);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_c);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_d);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_e);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_f);
		exgam=gam;
	}

	if (lum!=exlum)
	{
		// Luminosité
		lum_b[2]=lum;
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)lum_a);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)lum_b);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)lum_c);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)lum_d);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)lum_e);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)lum_f);
		exlum=lum;
	}

	if (net!=exnet)
	{
		// Netteté
		nettete[1]=(unsigned char)net;
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0032, 3, (void*)nettete);
		exnet=net;
	}

	return 0;
}


/** 
 * @param dev Device structure
 * 
 * @returns 0 if all is OK
 *
 * @brief This function permits to modify the settings of the camera.
 *
 * This functions permits to modify the settings :
 *   - brightness
 *   - contrast
 *   - white balance
 *   - ...
 *
 */
int dev_0503his_set_camera_quality(struct usb_gl860 *dev)
{
	return 0;
}


/** 
 * @param dev Device structure
 * 
 * @returns 0 if all is OK
 *
 * @brief This function permits to modify the settings of the camera.
 *
 * This functions permits to modify the frame rate per second.
 *
 */
int dev_0503his_set_camera_fps(struct usb_gl860 *dev)
{
	return 0;
}


/** 
 * @param dev Device structure
 * 
 * @returns 0 if all is OK
 *
 * @brief This function sets the device to start the stream.
 *
 * After the initialization of the device and the initialization of the video stream,
 * this function permits to enable the stream.
 */
int dev_0503his_start_stream(struct usb_gl860 *dev)
{
	STK_DEBUG("dev_0503his_start_stream [ini]\n");

	exlum=-1;
	exmirh=-1;
	exmirv=-1;
	exnet=-1;
	excntr=-1;
	excontrej=-1;
	exfreq=-1;
	exgam=-1;

	msleep(300);

   ctrl_out(dev, 0x40, 5, 0x0001, 0x0000, 0, NULL); msleep(35);

	commun(dev);

//#ifdef _0503A_
if (_0503A_)
{
	ctrl_out(dev, 0x40,  1, 0x0041, 0x0000,  0, NULL); msleep(70);
   ctrl_out(dev, 0x40,  1, 0x0040, 0x0000,  0, NULL);
	ctrl_out(dev, 0x40,  1, 0x0010, 0x0010,  0, NULL);
	ctrl_out(dev, 0x40,  1, 0x0003, 0x00c1,  0, NULL);
	ctrl_out(dev, 0x40,  1, 0x0042, 0x00c2,  0, NULL);
	ctrl_out(dev, 0x40,  1, 0x006a, 0x000d,  0, NULL);
}
//#else
{
	ctrl_out(dev, 0x40,  1, 0x0041, 0x0000,  0, NULL); msleep(70);
	ctrl_out(dev, 0x40,  1, 0x0010, 0x0010,  0, NULL);
	ctrl_out(dev, 0x40,  1, 0x0003, 0x00c1,  0, NULL);
	ctrl_out(dev, 0x40,  1, 0x0042, 0x00c2,  0, NULL);
	ctrl_out(dev, 0x40,  1, 0x006a, 0x000d,  0, NULL);
	}
//#endif

   switch (dev->resolution) {
      case WCAM_800x600:
      case WCAM_640x480:
      case WCAM_320x240:
      case WCAM_160x120:
      case WCAM_213x160:
      case WCAM_128x96:
      case WCAM_80x60:
			if (dev->resolution!=WCAM_800x600)
				ctrl_out(dev, 0x40,  3, 0x0000, 0x0200, 12, "\xd0\x02\xd1\x08\xd2\xe1\xd3\x02\xd4\x10\xd5\x81");
			else
				ctrl_out(dev, 0x40,  3, 0x0000, 0x0200, 12, "\xd0\x02\xd1\x10\xd2\x57\xd3\x02\xd4\x18\xd5\x21");

//		#ifdef _0503B_
		if (_0503B_)
		{
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x15");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x25");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x22\x2e");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x81");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa4\x08");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x17");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa4\x09");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x1a");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa4\x0a");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x1d");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa4\x0b");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x20");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x24\x11");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x81");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x24\x13");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x9b");
		}
//		#endif

			if (dev->resolution==WCAM_800x600)
			{
				ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x03");
				ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x03\x24");
				ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x05");
				ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x02\x58");
				ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03");
				ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x05");
				ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03");
				ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06");
			}

			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\x1e\x8f\x09");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\x1c\x01\x28");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\x1e\x8f\x09");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\x1e\x8f\x09");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0032,  3, "\x14\x06\xe6");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x20");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x01");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x2e\x01\x00");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\x04\x00\x2a");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa7\x02");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x95");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x01\x00");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x20");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x72");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x02");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa7\x02");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x01");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x20");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00"); msleep(53);
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x01");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa7\x02");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x05"); msleep(53);
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x05");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06"); msleep(53);

		if (_0503A_)
//		#if defined(_0503A_)
		{
			ctrl_out(dev, 0x40, 1, 0x0001, 0x0010, 0, NULL);
			ctrl_out(dev, 0x40, 1, 0x0000, 0x00c1, 0, NULL);
			ctrl_out(dev, 0x40, 1, 0x0041, 0x00c2, 0, NULL); msleep(120);
		}
		else if (_0503B_)
//		#elif defined(_0503B_)
		{
			ctrl_out(dev, 0x40, 1, 0x0010, 0x0010, 0, NULL);
			ctrl_out(dev, 0x40, 1, 0x0000, 0x00c1, 0, NULL);
			ctrl_out(dev, 0x40, 1, 0x0041, 0x00c2, 0, NULL); msleep(120);
			ctrl_out(dev, 0x40, 1, 0x0040, 0x0000, 0, NULL); msleep(30);
		}
		else if (_0503C_)
//		#elif defined(_0503C_)
		{
			ctrl_out(dev, 0x40, 1, 0x0001, 0x0010, 0, NULL);
			ctrl_out(dev, 0x40, 1, 0x0000, 0x00c1, 0, NULL);
			ctrl_out(dev, 0x40, 1, 0x0041, 0x00c2, 0, NULL); msleep(120);
			ctrl_out(dev, 0x40, 1, 0x0040, 0x0000, 0, NULL); msleep(30);
		}
//		#endif

			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa4\x04");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\xc0"); msleep(20);
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa7\x9d");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa7\x9e");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa2\x0c");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x17");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x05");

//		#ifdef _0503B_
		if (_0503B_)
		{
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0032,  3, "\x10\x01\xf8");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xce\x01\xa8");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xd0\x66\x33");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xd2\x31\x9a");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xd4\x94\x63");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xd6\x4b\x25");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xd8\x26\x70");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xda\x72\x4c");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xdc\xff\x04");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xde\x01\x5b");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xe6\x01\x13");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xee\x0b\xf0");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xf6\x0b\xa4");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x00\xf6\xe7");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x08\x0d\xfd");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x10\x25\x63");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x18\x35\x6c");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x20\x42\x7e");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x28\x19\x44");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x30\x39\xd4");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x38\xf5\xa8");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x4c\x07\x90");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x44\x07\xb8");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x5c\x06\x88");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x54\x07\xff");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xe0\x01\x52");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xe8\x00\xcc");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xf0\x0d\x83");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xf8\x0c\xb3");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x02\xfe\xba");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x0a\x04\xe0");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x12\x1c\x63");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x1a\x2b\x5a");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x22\x32\x5e");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x2a\x0d\x28");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x32\x2c\x02");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x3a\xf4\xfa");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x4e\x07\xef");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x46\x07\x88");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x5e\x07\xc1");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x56\x04\x64");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xe4\x01\x15");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xec\x00\x82");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xf4\x0c\xce");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xfc\x0c\xba");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x06\x1f\x02");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x0e\x02\xe3");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x16\x1a\x50");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x1e\x24\x39");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x26\x23\x4c");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x2e\xf9\x1b");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x36\x23\x19");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x3e\x12\x08");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x52\x07\x22");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x4a\x03\xd3");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x62\x06\x54");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x5a\x04\x5d");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xe2\x01\x04");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xea\x00\xa0");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xf2\x0c\xbc");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xfa\x0c\x5b");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x04\x17\xf2");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x0c\x02\x08");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x14\x28\x43");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x1c\x28\x62");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x24\x2b\x60");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x2c\x07\x33");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x34\x1f\xb0");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x3c\xed\xcd");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x50\x00\x06");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x48\x07\xff");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x60\x05\x89");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x58\x07\xff");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x40\x00\xa0");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x42\x00\x00");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0032,  3, "\x10\x01\xfc");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x18");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x3c");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x19");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x04\x6c");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x3b");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x24");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x02");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x1f");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x02");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x1f");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x02");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x1f");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x02");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x1f");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa2\x06");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x46");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x02");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x0f");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x05");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0032,  3, "\x6c\x14\x08");
		}
//		#endif
			break;

      case WCAM_1280x960:
      case WCAM_1600x1200:
			if (dev->resolution==WCAM_1280x960)
			{
				ctrl_out(dev, 0x40,  3, 0x0000, 0x0200, 12, "\xd0\x02\xd1\x20\xd2\x01\xd3\x02\xd4\x28\xd5\x01");
				ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x07");
				ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x05\x04");
				ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x09");
				ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x04\x02");
			}
			else
			{
				ctrl_out(dev, 0x40,  3, 0x0000, 0x0200, 12, "\xd0\x02\xd1\x20\xd2\xaf\xd3\x02\xd4\x30\xd5\x41");
				ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x07");
				ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x06\x40");
				ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x09");
				ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x04\xb0");
			}

			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x05");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\x1e\x8f\x09");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\x1c\x01\x28");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\x1e\x8f\x09");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\x1e\x8f\x09");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0032,  3, "\x14\x06\xe6");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x05");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x05");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x20");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x72");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x30");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x03");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x31");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x02");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x32");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x03");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x34");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x03");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x02");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x2e\x01\x00");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\x04\x00\x2a");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa7\x02");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x01");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x97");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x01\x00"); msleep(53);
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x20");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x01");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa7\x02");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00"); msleep(53);
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x20");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x72");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x02");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa7\x02");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x01"); msleep(53);

//		#if defined(_0503A_)
		if (_0503A_)
		{
			ctrl_out(dev, 0x40, 1, 0x0001, 0x0010, 0, NULL);
			ctrl_out(dev, 0x40, 1, 0x0000, 0x00c1, 0, NULL);
			ctrl_out(dev, 0x40, 1, 0x0041, 0x00c2, 0, NULL); msleep(120);
		}
		else if (_0503B_ || _0503C_)
		{
//		#elif defined(_0503B_) || defined(_0503C_)
			ctrl_out(dev, 0x40, 1, 0x0001, 0x0010, 0, NULL);
			ctrl_out(dev, 0x40, 1, 0x0000, 0x00c1, 0, NULL);
			ctrl_out(dev, 0x40, 1, 0x0041, 0x00c2, 0, NULL); msleep(120);
			ctrl_out(dev, 0x40, 1, 0x0040, 0x0000, 0, NULL); msleep(30);
		}
//		#endif

			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa4\x04");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\xc0"); msleep(20);
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa7\x9d");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa7\x9e");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa2\x0c");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x17");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x05");

//		#ifdef _0503B_
		if (_0503B_)
		{
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0032,  3, "\x10\x01\xf8");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xce\x01\xa8");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xd0\x66\x33");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xd2\x31\x9a");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xd4\x94\x63");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xd6\x4b\x25");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xd8\x26\x70");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xda\x72\x4c");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xdc\xff\x04");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xde\x01\x5b");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xe6\x01\x13");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xee\x0b\xf0");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xf6\x0b\xa4");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x00\xf6\xe7");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x08\x0d\xfd");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x10\x25\x63");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x18\x35\x6c");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x20\x42\x7e");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x28\x19\x44");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x30\x39\xd4");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x38\xf5\xa8");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x4c\x07\x90");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x44\x07\xb8");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x5c\x06\x88");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x54\x07\xff");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xe0\x01\x52");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xe8\x00\xcc");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xf0\x0d\x83");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xf8\x0c\xb3");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x02\xfe\xba");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x0a\x04\xe0");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x12\x1c\x63");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x1a\x2b\x5a");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x22\x32\x5e");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x2a\x0d\x28");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x32\x2c\x02");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x3a\xf4\xfa");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x4e\x07\xef");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x46\x07\x88");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x5e\x07\xc1");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x56\x04\x64");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xe4\x01\x15");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xec\x00\x82");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xf4\x0c\xce");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xfc\x0c\xba");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x06\x1f\x02");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x0e\x02\xe3");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x16\x1a\x50");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x1e\x24\x39");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x26\x23\x4c");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x2e\xf9\x1b");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x36\x23\x19");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x3e\x12\x08");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x52\x07\x22");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x4a\x03\xd3");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x62\x06\x54");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x5a\x04\x5d");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xe2\x01\x04");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xea\x00\xa0");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xf2\x0c\xbc");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xfa\x0c\x5b");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x04\x17\xf2");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x0c\x02\x08");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x14\x28\x43");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x1c\x28\x62");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x24\x2b\x60");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x2c\x07\x33");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x34\x1f\xb0");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x3c\xed\xcd");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x50\x00\x06");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x48\x07\xff");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x60\x05\x89");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x58\x07\xff");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x40\x00\xa0");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x42\x00\x00");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0032,  3, "\x10\x01\xfc");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x18");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x3c");
		}
//		#endif

			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x19");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x04\x6c");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x3b");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x24");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x01");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x20");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa7\x02");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x02");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x20");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x72");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa7\x02");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x01");

		//#ifdef _0503B_
		if (_0503B_)
		{
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x02");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x1f");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x02");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x1f");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x02");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x1f");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x02");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x1f");
		}
//		#endif
	}

	STK_DEBUG("dev_0503his_start_stream [fini]\n");
	return 0;
}


/** 
 * @param dev Device structure
 * 
 * @returns 0 if all is OK
 *
 * @brief Reconfigure the camera before the stream.
 *
 * Before enabling the video stream, you have to reconfigure the device.
 */
int dev_0503his_reconf_camera(struct usb_gl860 *dev)
{
   switch (dev->resolution) {
      case WCAM_640x480:
      case WCAM_320x240:
      case WCAM_160x120:
      case WCAM_213x160:
      case WCAM_128x96:
      case WCAM_80x60:
         dev->lealt=3;
         dev->offset=2048;
         dev->nbPaq=32;
         break;

      case WCAM_800x600:
         dev->lealt=1;
         dev->offset=3072;
         dev->nbPaq=32;
         break;

      case WCAM_1280x960:
         dev->lealt=1;
         dev->offset=3072;
         dev->nbPaq=32;
         break;

      case WCAM_1600x1200:
         dev->lealt=1;
         dev->offset=3072;
         dev->nbPaq=32;
         break;
   }
	return 0;
}


/** 
 * @param dev Device structure
 * 
 * @returns 0 if all is OK
 *
 * @brief This function sets the device to stop the stream.
 *
 * You use the function start_stream to enable the video stream. So you
 * have to use the function stop_strem to disable the video stream.
 */
int dev_0503his_stop_stream(struct usb_gl860 *dev)
{
	STK_DEBUG("dev_0503his_stop_stream [ini]\n");

//#if defined (_0503B_) || defined (_0503C_)
if (_0503B_ || _0503C_) {
   ctrl_out(dev, 0x40, 5, 0x0000, 0x0000, 0, NULL);
   msleep(20);
   ctrl_out(dev, 0x40, 1, 0x0001, 0x0000, 0, NULL);
} else {
//#else
   ctrl_out(dev, 0x40, 5, 0x0000, 0x0000, 0, NULL);
   msleep(20);
   ctrl_out(dev, 0x40, 1, 0x0041, 0x0000, 0, NULL);
}
//#endif

	STK_DEBUG("dev_0503his_stop_stream [fini]\n");
	return 0;
}

