/** 
 * @file gl860-dev.c
 * @author Olivier LORIN, tiré du pilote Syntek par Nicolas VIVIEN
 * @date 2008-05-05
 * @version v0.9.0
 *
 * @brief Driver for GL860 USB video camera
 *
 * @note Copyright (C) Olivier LORIN
 *
 * @par Licences
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <linux/module.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/version.h>
#include <linux/errno.h>
#include <linux/slab.h>
#include <linux/kref.h>

#include <linux/usb.h>
#include <media/v4l2-common.h>

#include "gl860.h"
#include "gl860-dev.h"

#define ctrl_in fct_RTx
#define ctrl_out fct_RTx

//#ifndef _0503SIM_
static int nbretournee=3;
//#endif

int (*dev_initialize_device)(struct usb_gl860 *);
int (*dev_configure_device)(struct usb_gl860 *, int);
int (*dev_start_stream)(struct usb_gl860 *);
int (*dev_stop_stream)(struct usb_gl860 *);
int (*dev_camera_asleep)(struct usb_gl860 *);
int (*dev_init_camera)(struct usb_gl860 *);
int (*dev_sensor_settings)(struct usb_gl860 *);
int (*dev_reconf_camera)(struct usb_gl860 *);
int (*dev_camera_settings)(struct usb_gl860 *);
int (*dev_set_camera_quality)(struct usb_gl860 *);
int (*dev_set_camera_fps)(struct usb_gl860 *);

void dev_gl860_devinette(struct usb_gl860 *dev)
{
   unsigned char sondea1, sondea2, sondea3, sondeb1, sondeb2, sondeb3;

   STK_DEBUG("dev_0503his_initialize_device [ini]\n");

   ctrl_in(dev, 0xc0, 2, 0x0000, 0x0004, 1, &sondea1);
   ctrl_in(dev, 0xc0, 2, 0x0000, 0x0004, 1, &sondea1);

   ctrl_out(dev, 0x40, 1, 0x0000, 0x0000, 0, NULL); msleep(53);
   ctrl_out(dev, 0x40, 1, 0x0010, 0x0010, 0, NULL); msleep(53);
   ctrl_out(dev, 0x40, 1, 0x0008, 0x00c0, 0, NULL); msleep(53);
   ctrl_out(dev, 0x40, 1, 0x0001, 0x00c1, 0, NULL); msleep(53);
   ctrl_out(dev, 0x40, 1, 0x0001, 0x00c2, 0, NULL); msleep(53);
   ctrl_out(dev, 0x40, 1, 0x0020, 0x0006, 0, NULL); msleep(53);
   ctrl_out(dev, 0x40, 1, 0x006a, 0x000d, 0, NULL); msleep(53);

   ctrl_out(dev, 0x40, 1, 0x0040, 0x0000, 0, NULL); msleep(5);
   ctrl_out(dev, 0x40, 1, 0x0063, 0x0006, 0, NULL); msleep(5);
   ctrl_out(dev, 0x40, 1, 0x7a00, 0x8030, 0, NULL);
   ctrl_in(dev, 0xc0, 2, 0x7a00, 0x8030, 1, &sondea1);
   ctrl_out(dev, 0x40, 1, 0x0040, 0x0000, 0, NULL); msleep(5);
   ctrl_out(dev, 0x40, 1, 0x0063, 0x0006, 0, NULL); msleep(5);
   ctrl_out(dev, 0x40, 1, 0x7a00, 0x8030, 0, NULL);
   ctrl_in(dev, 0xc0, 2, 0x7a00, 0x8030, 1, &sondea2);
   ctrl_out(dev, 0x40, 1, 0x0040, 0x0000, 0, NULL); msleep(5);
   ctrl_out(dev, 0x40, 1, 0x0063, 0x0006, 0, NULL); msleep(5);
   ctrl_out(dev, 0x40, 1, 0x7a00, 0x8030, 0, NULL);
   ctrl_in(dev, 0xc0, 2, 0x7a00, 0x8030, 1, &sondea3);
	if (sondea3==sondea2 || sondea3==sondea1) sondea1=sondea3;
   STK_INFO("probe 1=%2x\n",sondea1);

   ctrl_in(dev, 0xc0,  2, 0x0000, 0x0000,  1, &sondeb1);
   ctrl_in(dev, 0xc0,  2, 0x0000, 0x0000,  1, &sondeb2);
   ctrl_in(dev, 0xc0,  2, 0x0000, 0x0000,  1, &sondeb3);
	if (sondeb3==sondeb2 || sondeb3==sondeb1) sondeb1=sondeb3;
	if (sondeb1==0x40) sondeb1=0x50;
	if (sondeb1==0xc8) sondeb1=0xd8;
	if (sondeb1!=0x50 && sondeb1!=0xd8) sondeb1=0x76;
   STK_INFO("probe 2=%2x\n",sondeb1);

	if (sondea1==0x00 && sondeb1==0x50)
	{
		dev->pilote=ID0503HS;
		STK_INFO("Version Hulkie/Soro\n");
		dev_initialize_device  = dev_0503his_initialize_device;
		dev_configure_device   = dev_0503his_configure_device;
		dev_start_stream       = dev_0503his_start_stream;
		dev_stop_stream        = dev_0503his_stop_stream;
		dev_camera_asleep      = dev_0503his_camera_asleep;
		dev_init_camera        = dev_0503his_init_camera;
		dev_sensor_settings    = dev_0503his_sensor_settings;
		dev_reconf_camera      = dev_0503his_reconf_camera;
		dev_camera_settings    = dev_0503his_camera_settings;
		dev_set_camera_quality = dev_0503his_set_camera_quality;
		dev_set_camera_fps     = dev_0503his_set_camera_fps;
	}
	else
	if (sondea1==0x00 && sondeb1==0xd8)
	{
		dev->pilote=ID0503IM;
		STK_INFO("Version Iceman\n");
		dev_initialize_device  = dev_0503his_initialize_device;
		dev_configure_device   = dev_0503his_configure_device;
		dev_start_stream       = dev_0503his_start_stream;
		dev_stop_stream        = dev_0503his_stop_stream;
		dev_camera_asleep      = dev_0503his_camera_asleep;
		dev_init_camera        = dev_0503his_init_camera;
		dev_sensor_settings    = dev_0503his_sensor_settings;
		dev_reconf_camera      = dev_0503his_reconf_camera;
		dev_camera_settings    = dev_0503his_camera_settings;
		dev_set_camera_quality = dev_0503his_set_camera_quality;
		dev_set_camera_fps     = dev_0503his_set_camera_fps;
	}
	else
	if (sondea1==0xff && sondeb1==0x50)
	{
		dev->pilote=ID0503MS;
		STK_INFO("Version Malmostoso\n");
		dev_initialize_device  = dev_0503ms_initialize_device;
		dev_configure_device   = dev_0503ms_configure_device;
		dev_start_stream       = dev_0503ms_start_stream;
		dev_stop_stream        = dev_0503ms_stop_stream;
		dev_camera_asleep      = dev_0503ms_camera_asleep;
		dev_init_camera        = dev_0503ms_init_camera;
		dev_sensor_settings    = dev_0503ms_sensor_settings;
		dev_reconf_camera      = dev_0503ms_reconf_camera;
		dev_camera_settings    = dev_0503ms_camera_settings;
		dev_set_camera_quality = dev_0503ms_set_camera_quality;
		dev_set_camera_fps     = dev_0503ms_set_camera_fps;
	}
	else
	if (sondea1==0xff && sondeb1==0xd8)
	{
		dev->pilote=IDF191;
		STK_INFO("Version F191\n");
		dev_initialize_device  = dev_f191_initialize_device;
		dev_configure_device   = dev_f191_configure_device;
		dev_start_stream       = dev_f191_start_stream;
		dev_stop_stream        = dev_f191_stop_stream;
		dev_camera_asleep      = dev_f191_camera_asleep;
		dev_init_camera        = dev_f191_init_camera;
		dev_sensor_settings    = dev_f191_sensor_settings;
		dev_reconf_camera      = dev_f191_reconf_camera;
		dev_camera_settings    = dev_f191_camera_settings;
		dev_set_camera_quality = dev_f191_set_camera_quality;
		dev_set_camera_fps     = dev_f191_set_camera_fps;
	}
	else
	{
		dev->pilote=ID0503SIM;
		dev_initialize_device  = dev_0503sim_initialize_device;
		dev_configure_device   = dev_0503sim_configure_device;
		dev_start_stream       = dev_0503sim_start_stream;
		dev_stop_stream        = dev_0503sim_stop_stream;
		dev_camera_asleep      = dev_0503sim_camera_asleep;
		dev_init_camera        = dev_0503sim_init_camera;
		dev_sensor_settings    = dev_0503sim_sensor_settings;
		dev_reconf_camera      = dev_0503sim_reconf_camera;
		dev_camera_settings    = dev_0503sim_camera_settings;
		dev_set_camera_quality = dev_0503sim_set_camera_quality;
		dev_set_camera_fps     = dev_0503sim_set_camera_fps;
	}

}

/** 
 * @param dev Device structure
 * 
 * @returns 0 if all is OK
 *
 * @brief This function permits to initialize the device.
 *
 * This function must be called at first. It's the start of the
 * initialization process. After this process, the device is
 * completly initalized and it's ready.
 *
 * This function is written from the USB log.
 */
int dev_gl860_initialize_device(struct usb_gl860 *dev)
{
	int ret;

//	dev_gl860_devinette(dev);

	ret = dev_initialize_device(dev);
/*
	switch (dev->webcam_model) {
		case GENESYS_F191:
			ret = dev_f191_initialize_device(dev);
			break;

		case GENESYS_0503:
			ret = dev_0503_initialize_device(dev);
			break;

		default:
			ret = -1;
	}
*/
	return ret;
}


/** 
 * @param dev Device structure
 * 
 * @returns 0 if all is OK
 *
 * @brief This function initializes the device for the stream.
 *
 * It's the start. This function has to be called at first, before
 * enabling the video stream.
 */
int dev_gl860_init_camera(struct usb_gl860 *dev)
{
	int ret;
	ret = dev_init_camera(dev);
/*
	switch (dev->webcam_model) {
		case GENESYS_F191:
			ret = dev_f191_init_camera(dev);
			break;

		case GENESYS_0503:
			ret = dev_0503_init_camera(dev);
			break;

		default:
			ret = -1;
	}
*/
	return ret;
}


/** 
 * @param dev Device structure
 * @param nbr Number of tries
 * 
 * @returns 0 if all is OK
 *
 * @brief This function permits to check the device in reading the register 0x0201.
 *
 * When we configure the gl860, this function is used to check the device status.
 *   - If the read value is 0x00, then the device isn't ready.
 *   - If the read value is 0x04, then the device is ready.
 *   - If the read value is other, then the device is misconfigured.
 */
/*
int dev_gl860_check_device(struct usb_gl860 *dev, int nbr)
{
	int i;
	int value;

	for (i=0; i<nbr; i++) {
//		usb_gl860_read_registry(dev, 0x201, &value);
		
		if (value == 0x00) {
		}
		else if ((value == 0x14) || (value == 0x30) || (value == 0x70) || (value == 0x71)) {
		}
		else if (value == 0x01) {
			return 1;
		}
		else if ((value == 0x04) || (value == 0x05))
			return 1;
		else if (value == 0x15) 
			return 1;
		else {
			STK_ERROR("Check device return error (0x0201 = %02X) !\n", value);
			return -1;
		}
	}

	return 0;
}
*/

/** 
 * @param dev Device structure
 * 
 * @returns Value of register 0x0001
 *
 * @brief A espece of software watchdog.
 *
 * This function reads periodically the value of register 0x0001.
 * 
 * We don't know the purpose. I assume that it seems to a software watchdog.
 */
int dev_gl860_watchdog_camera(struct usb_gl860 *dev)
{
	int value=0;

	//#ifndef _0503SIM_
	if (!_0503SIM_)
	{
		unsigned char etat;

	   ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)&etat);

		if (etat==0xc8 || etat==0x40) dev->vsettings.retournee=1; else dev->vsettings.retournee=0;
		if (dev->vsettings.retournee) nbretournee=0;//{ nbretournee=0; en_retourne=1; }
		if (!dev->vsettings.retournee && nbretournee<3) nbretournee++;
		if (nbretournee==3) { dev->vsettings.hflip=0; dev->vsettings.vflip=0; } else { dev->vsettings.hflip=1; dev->vsettings.vflip=1; }//en_retourne=0;
//		STK_INFO("watchdog=%02x\n",etat&0xFF);

		switch (dev->webcam_model) {
			case GENESYS_F191:
//				dev_f191_camera_settings(dev);
				break;

			case GENESYS_0503:
//				dev_0503_camera_settings(dev);
				break;
		}
	}
	//#endif

//	usb_gl860_read_registry(dev, 0x0001, &value);
//
//	if (value != 0x03) {
//		STK_ERROR("Error : Register 0x0001 = %02X\n", value);
//	}

	return value;
}


/** 
 * @param dev Device structure
 * 
 * @returns 0 if all is OK
 *
 * @brief This function switchs on the camera.
 *
 * In fact, we choose the alternate interface '3'.
 */
int dev_gl860_camera_on(struct usb_gl860 *dev)
{
	int ret = -1;
	struct usb_device *udev = dev->udev;

	dev->cam_off=0;
	dev->err45=0;
//	nbretournee=3;

	STK_DEBUG("dev_gl860_camera_on [ini]\n");

	ret = usb_set_interface(udev, 0, dev->lealt);

	if (ret < 0)
		STK_ERROR("usb_set_interface failed !\n");

	STK_DEBUG("dev_gl860_camera_on [fini]\n");
	return ret;
}


/** 
 * @param dev Device structure
 * 
 * @returns 0 if all is OK
 *
 * @brief This function switchs off the camera.
 *
 * In fact, we choose the alternate interface '0'.
 */
int dev_gl860_camera_off(struct usb_gl860 *dev)
{
	int ret = -1;
	struct usb_device *udev = dev->udev;

	dev->cam_off=1;

	STK_DEBUG("dev_gl860_camera_off [ini]\n");

	ret = usb_set_interface(udev, 0, 0);

	if (ret < 0)
		STK_ERROR("usb_set_interface failed !\n");

	STK_DEBUG("dev_gl860_camera_off [fini]\n");

	dev->cam_off=0;

	return 0;
}


/** 
 * @param dev Device structure
 * 
 * @returns 0 if all is OK
 *
 * @brief Wake-up the camera.
 *
 * This function permits to wake-up the device.
 */
int dev_gl860_camera_asleep(struct usb_gl860 *dev)
{
	int ret;

	ret = dev_camera_asleep(dev);
/*
	switch (dev->webcam_model) {
		case GENESYS_F191:
			ret = dev_f191_camera_asleep(dev);
			break;

		case GENESYS_0503:
			ret = dev_0503_camera_asleep(dev);
			break;

		default:
			ret = -1;
	}
*/
	return ret;
}


/** 
 * @param dev Device structure
 * 
 * @returns 0 if all is OK
 *
 * @brief This function permits to modify the settings of the camera.
 *
 * This functions permits to modify the settings :
 *   - brightness
 *   - contrast
 *   - white balance
 *   - ...
 */
int dev_gl860_camera_settings(struct usb_gl860 *dev)
{
	int ret;

	ret = dev_camera_settings(dev);
/*
	switch (dev->webcam_model) {
		case GENESYS_F191:
			ret = dev_f191_camera_settings(dev);
			break;

		case GENESYS_0503:
			ret = dev_0503_camera_settings(dev);
			break;

		default:
			ret = -1;
	}
*/
	return ret;
}


/** 
 * @param dev Device structure
 * 
 * @returns 0 if all is OK
 *
 * @brief This function permits to modify the quality video of the camera.
 *
 * This functions permits to modify the settings :
 *   - brightness
 *   - contrast
 *   - white balance
 *   - ...
 */
int dev_gl860_set_camera_quality(struct usb_gl860 *dev)
{
	int ret;

	ret = dev_set_camera_quality(dev);
/*
	switch (dev->webcam_model) {
		case GENESYS_F191:
			ret = dev_f191_set_camera_quality(dev);
			break;

		case GENESYS_0503:
			ret = dev_0503_set_camera_quality(dev);
			break;

		default:
			ret = -1;
	}
*/
	return ret;
}


/** 
 * @param dev Device structure
 * 
 * @returns 0 if all is OK
 *
 * @brief This function permits to modify the fps of the camera.
 *
 * This functions permits to modify the frame rate per second of the camera.
 * So the number of images per second.
 */
int dev_gl860_set_camera_fps(struct usb_gl860 *dev)
{
	int ret;

	ret = dev_set_camera_fps(dev);
/*
	switch (dev->webcam_model) {
		case GENESYS_F191:
			ret = dev_f191_set_camera_fps(dev);
			break;

		case GENESYS_0503:
			ret = dev_0503_set_camera_fps(dev);
			break;

		default:
			ret = -1;
	}
*/
	return ret;
}


/** 
 * @param dev Device structure
 * 
 * @returns 0 if all is OK
 *
 * @brief This function sets the device to start the stream.
 *
 * After the initialization of the device and the initialization of the video stream,
 * this function permits to enable the stream.
 */
int dev_gl860_start_stream(struct usb_gl860 *dev)
{
	int ret;

	ret = dev_start_stream(dev);
/*
	switch (dev->webcam_model) {
		case GENESYS_F191:
			ret = dev_f191_start_stream(dev);
			break;

		case GENESYS_0503:
			ret = dev_0503_start_stream(dev);
			break;

		default:
			ret = -1;
	}
*/
	return ret;
}


/** 
 * @param dev Device structure
 * 
 * @returns 0 if all is OK
 *
 * @brief Reconfigure the camera before the stream.
 *
 * Before enabling the video stream, you have to reconfigure the device.
 */
int dev_gl860_reconf_camera(struct usb_gl860 *dev)
{
	int ret;

	ret = dev_reconf_camera(dev);
/*
	switch (dev->webcam_model) {
		case GENESYS_F191:
			ret = dev_f191_reconf_camera(dev);
			break;

		case GENESYS_0503:
			ret = dev_0503_reconf_camera(dev);
			break;

		default:
			ret = -1;
	}
*/
	return ret;
}


/** 
 * @param dev Device structure
 * 
 * @returns 0 if all is OK
 *
 * @brief This function sets the device to stop the stream.
 *
 * You use the function start_stream to enable the video stream. So you
 * have to use the function stop_strem to disable the video stream.
 */
int dev_gl860_stop_stream(struct usb_gl860 *dev)
{
	int ret;

	ret = dev_stop_stream(dev);
/*
	switch (dev->webcam_model) {
		case GENESYS_F191:
			ret = dev_f191_stop_stream(dev);
			break;

		case GENESYS_0503:
			ret = dev_0503_stop_stream(dev);
			break;

		default:
			ret = -1;
	}
*/
	return ret;
}

