/*
 * Driver for the unknown_0xdc sensor.
 * This is probably a pixel plus sensor but we haven't identified it yet
 *
 * Copyright (c) 2008 Erik Andrén
 * Copyright (c) 2007 Ilyes Gouta. Based on the m5603x Linux Driver Project.
 * Copyright (Cc) 2005 m5603x Linux Driver Project <m5602@x3ng.com.br>
 *
 * Portions of code to USB interface and ALi driver sofwtare,
 * Copyright (c) 2006 Willem Duinker
 * v4l2 interface modeled after the V4L2 driver
 * for SN9C10x PC Camera Controllers
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, version 2.
 *
 */

#ifndef M5602_UNKNOWN_0xdc_H_
#define M5602_UNKNOWN_0xdc_H_

#include "m5602.h"

/*****************************************************************************/

#define SENSOR_HIGH_EXPOSURE 	0x1a
#define SENSOR_LOW_EXPOSURE  	0x1b
#define SENSOR_CLARITY 	     	0x1c
#define SENSOR_GAIN 	     	0x15

#define SENSOR_GAIN_DEFAULT  	0x12

#define SENSOR_EXPOSURE_DEFAULT 0xf0ff

/*****************************************************************************/

static int unknown_0xdc_get_ctrl(struct m5602_camera *cam,
			   struct v4l2_control *ctrl);
static int unknown_0xdc_set_ctrl(struct m5602_camera *cam,
			   const struct v4l2_control *ctrl);
static int unknown_0xdc_probe(struct m5602_camera *cam);
static int unknown_0xdc_init(struct m5602_camera *cam);

static void unknown_0xdc_dump_registers(struct m5602_camera *cam);

static int unknown_0xdc_read_sensor(struct m5602_camera *cam, const u8 address, u8 *i2c_data, const u8 len);
static int unknown_0xdc_write_sensor(struct m5602_camera *cam, const u8 address, u8 *i2c_data, const u8 len);

static struct m5602_sensor unknown_0xdc = {
	.name = "UNKNOWN_0XDC",

	.i2c_slave_id = 0xdc,

	.probe = unknown_0xdc_probe,
	.init = unknown_0xdc_init,
	.get_ctrl = unknown_0xdc_get_ctrl,
	.set_ctrl = unknown_0xdc_set_ctrl,
	.qctrl = {
		{
			.id = V4L2_CID_GAIN,
			.type = V4L2_CTRL_TYPE_INTEGER,
			.name = "gain",
			.minimum = 0x00,
			.maximum = 0xff,
			.step = 0x1,
			.default_value = SENSOR_GAIN_DEFAULT
		},

		{
			.id = V4L2_CID_EXPOSURE,
			.type = V4L2_CTRL_TYPE_INTEGER,
			.name = "exposure",
			.minimum = 0x00,
			.maximum = 0xffff,
			.step = 0x1,
			.default_value = SENSOR_EXPOSURE_DEFAULT
		},
	},

	.cropcap = {
		.bounds = {
			.left = 0,
			.top = 0,
			.width = M5602_DEFAULT_FRAME_WIDTH,
			.height = M5602_DEFAULT_FRAME_HEIGHT
		},
		.defrect = {
			.left = 0,
			.top = 0,
			.width = M5602_DEFAULT_FRAME_WIDTH,
			.height = M5602_DEFAULT_FRAME_HEIGHT
  		}
	}
};

static const unsigned char preinit_unknown_0xdc[][3] =
{
	{BRIDGE, 0x13, 0x02},
	{BRIDGE, 0x12, 0xb0},
	{BRIDGE, 0x15, 0x00},
	{BRIDGE, 0x14, 0xb0},
	{BRIDGE, 0x60, 0xc0},
	{BRIDGE, 0x00, 0x0d},
	{BRIDGE, 0x01, 0x00},
	{BRIDGE, 0x60, 0xc0},
	{BRIDGE, 0x00, 0x0c},
	{SENSOR, 0x3f, 0x24},
	{BRIDGE, 0x77, 0x05},
	{BRIDGE, 0x76, 0x04},
	{BRIDGE, 0x74, 0x06},
	{BRIDGE, 0x72, 0x06},
	{BRIDGE, 0x70, 0x02},
	{BRIDGE, 0x15, 0x04},
	{BRIDGE, 0x14, 0xb0},
	{BRIDGE, 0x15, 0x06},
	{BRIDGE, 0x14, 0xb0},
	{BRIDGE, 0x60, 0xc0},
	{BRIDGE, 0x00, 0x0c},
	{BRIDGE, 0x02, 0x81},
	{BRIDGE, 0x04, 0x82},
	{BRIDGE, 0x0a, 0x01},
	{BRIDGE, 0x06, 0x00},
	{BRIDGE, 0x06, 0x02},
	{BRIDGE, 0x06, 0x00},
	{BRIDGE, 0x06, 0x00},
	{BRIDGE, 0x06, 0x01},
	{BRIDGE, 0x06, 0xec},
	{BRIDGE, 0x06, 0x00},
	{BRIDGE, 0x06, 0x00},
	{BRIDGE, 0x0a, 0x00},
	{BRIDGE, 0x0a, 0x02},
	{BRIDGE, 0x07, 0x00},
	{BRIDGE, 0x07, 0x00},
	{BRIDGE, 0x07, 0x02},
	{BRIDGE, 0x07, 0x87},
	{BRIDGE, 0x0a, 0x00},
	{SENSOR, 0x3f, 0x24},
	{BRIDGE, 0x77, 0x05},
	{BRIDGE, 0x76, 0x04},
	{BRIDGE, 0x74, 0x06},
	{BRIDGE, 0x72, 0x06},
	{BRIDGE, 0x70, 0x02},
	{BRIDGE, 0x15, 0x04},
	{BRIDGE, 0x14, 0xb0},
        {BRIDGE, 0x15, 0x00},
        {BRIDGE, 0x14, 0xb0},
        {BRIDGE, 0x60, 0xc0},
        {BRIDGE, 0x00, 0x0c},
	{BRIDGE, 0x77, 0x05},
        {BRIDGE, 0x76, 0x00}
};

static const unsigned char init_unknown_0xdc[][4] =
{
	{BRIDGE, 0x13, 0x02},
	{BRIDGE, 0x12, 0xb0},
	/*sequence 1*/
	{BRIDGE, 0x15, 0x00},
	{BRIDGE, 0x14, 0xb0},
	{BRIDGE, 0x60, 0xc0},
	{BRIDGE, 0x00, 0x0d},

	{BRIDGE, 0x01, 0x00},
	{BRIDGE, 0x60, 0xc0},
	{BRIDGE, 0x00, 0x0c},
	/*end of sequence 1*/

	/*sequence 2 (same as stop sequence)*/
	{SENSOR, 0x3f, 0x24},
	{BRIDGE, 0x77, 0x05},
	{BRIDGE, 0x76, 0x04},
	{BRIDGE, 0x74, 0x06},
	{BRIDGE, 0x72, 0x06},
	{BRIDGE, 0x70, 0x02},

	{BRIDGE, 0x15, 0x04},
	{BRIDGE, 0x14, 0xb0},
	/*end of sequence 2*/

	/*sequence 5*/
	{BRIDGE, 0x15, 0x06},
	{BRIDGE, 0x14, 0xb0},
	{BRIDGE, 0x60, 0xc0},
	{BRIDGE, 0x00, 0x0c},
	{BRIDGE, 0x02, 0x81},
	{BRIDGE, 0x04, 0x82},
	{BRIDGE, 0x0a, 0x01},
	{BRIDGE, 0x06, 0x00},
	{BRIDGE, 0x06, 0x02},
	{BRIDGE, 0x06, 0x00},
	{BRIDGE, 0x06, 0x00},
	{BRIDGE, 0x06, 0x01},
	{BRIDGE, 0x06, 0xec},
	{BRIDGE, 0x06, 0x00},
	{BRIDGE, 0x06, 0x00},
	{BRIDGE, 0x0a, 0x00},
	{BRIDGE, 0x0a, 0x02},
	{BRIDGE, 0x07, 0x00},
	{BRIDGE, 0x07, 0x00},
	{BRIDGE, 0x07, 0x02},
	{BRIDGE, 0x07, 0x87},
	{BRIDGE, 0x0a, 0x00},
	/*end of sequence 5*/

	/*sequence 2 stop */
	{SENSOR, 0x3f, 0x24},

	{BRIDGE, 0x77, 0x05},
	{BRIDGE, 0x76, 0x04},
	{BRIDGE, 0x74, 0x06},
	{BRIDGE, 0x72, 0x06},
	{BRIDGE, 0x70, 0x02},
	{BRIDGE, 0x15, 0x04},
	{BRIDGE, 0x14, 0xb0},
	/*end of sequence 2 stop */

/* ---------------------------------
 * end of init - begin of start
 * --------------------------------- */

	/*sequence 3*/
	{BRIDGE, 0x15, 0x00},
	{BRIDGE, 0x14, 0xb0},
	{BRIDGE, 0x60, 0xc0},
	{BRIDGE, 0x00, 0x0c},
	/*end of sequence 3*/
	/*sequence 4*/
	{BRIDGE, 0x77, 0x05},
	{BRIDGE, 0x76, 0x00},
	{BRIDGE, 0x74, 0x06},
	{BRIDGE, 0x75, 0x00},

	{SENSOR, 0x3f, 0x04},
	{SENSOR, 0x04, 0x02},
	{SENSOR, 0x05, 0xef},
	{SENSOR, 0x06, 0x02},
	{SENSOR, 0x07, 0x1c},
	{SENSOR, 0x08, 0x00},
	{SENSOR, 0x09, 0x01},
	{SENSOR, 0x0a, 0x00},
	{SENSOR, 0x0b, 0x01},
	{SENSOR, 0x0c, 0x02},
	{SENSOR, 0x0d, 0x87},
	{SENSOR, 0x0e, 0x01},
	{SENSOR, 0x0f, 0xe3},
	{SENSOR, 0x5b, 0x04},
	{SENSOR, 0x5b, 0x04},
	{SENSOR, 0x3e, 0x08},
	{SENSOR, 0x1e, 0x03},
	{SENSOR, 0x21, 0x90},
	{SENSOR, 0x40, 0x60},
	{SENSOR, 0x59, 0x13},
	{SENSOR, 0x5a, 0x40},
	{SENSOR, 0x5f, 0x00},
	{SENSOR, 0x60, 0x80},
	{SENSOR, 0x78, 0x14},
	{SENSOR, 0x6f, 0x01},
	{SENSOR, 0x1d, 0x18},
	{SENSOR, 0x42, 0x14},
	{SENSOR, 0x63, 0x38},
	{SENSOR, 0x64, 0x38},
	{SENSOR, 0x1d, 0x58},
	{SENSOR, 0x16, 0x30},
	{SENSOR, 0x17, 0x30},
	{SENSOR, 0x18, 0x30},
	{SENSOR, 0x19, 0x30},
	{SENSOR, 0x2d, 0x10},
	{SENSOR, 0x2e, 0x20},
	{SENSOR, 0x2f, 0x40},
	{SENSOR, 0x30, 0x60},
	{SENSOR, 0x31, 0x80},
	{SENSOR, 0x32, 0xa0},
	{SENSOR, 0x33, 0xc0},
	{SENSOR, 0x34, 0xff},
	/*end of sequence 4*/
	/*sequence 5*/
	{BRIDGE, 0x15, 0x06},
	{BRIDGE, 0x14, 0xb0},
	{BRIDGE, 0x60, 0xc0},
	{BRIDGE, 0x00, 0x0c},
	{BRIDGE, 0x02, 0x81},
	{BRIDGE, 0x04, 0x82},
	{BRIDGE, 0x0a, 0x01},
	{BRIDGE, 0x06, 0x00},
	{BRIDGE, 0x06, 0x02},
	{BRIDGE, 0x06, 0x00},
	{BRIDGE, 0x06, 0x00},
	{BRIDGE, 0x06, 0x01},
	{BRIDGE, 0x06, 0xec},
	{BRIDGE, 0x06, 0x00},
	{BRIDGE, 0x06, 0x00},
	{BRIDGE, 0x0a, 0x00},
	{BRIDGE, 0x0a, 0x00},
	{BRIDGE, 0x07, 0x00},
	{BRIDGE, 0x07, 0x00},
	{BRIDGE, 0x07, 0x02},
	{BRIDGE, 0x07, 0x7e},
	{BRIDGE, 0x0a, 0x00},
	/*end of sequence 5*/

	/*sequence 6*/
	/* Changing 40 in f0 the image becomes green in bayer mode and red in rgb mode */
	{SENSOR, 0x16, 0x40},
	/* in changing 40 in f0 the image becomes green in bayer mode and red in rgb mode */
	{SENSOR, 0x18, 0x40},

	/* with a very low lighted environment increase the exposure but decrease the FPS (Frame Per Second) */
	{BRIDGE, 0x15, 0x00},
	{BRIDGE, 0x14, 0xb0},

	/* Controls high exposure more than  SENSOR_LOW_EXPOSURE, use only in low lighted environment (f0 is more than ff ?)*/
	{SENSOR, SENSOR_HIGH_EXPOSURE, 0xf0},

	/* Controls low exposure , use only in high lighted environment */
	{SENSOR, SENSOR_LOW_EXPOSURE, 0xff},

	/* Controls clarity (not sure) */
	{SENSOR, SENSOR_CLARITY, 0x00},
	/*  Controls gain (the image is more lighted) */
	{SENSOR, SENSOR_GAIN, 0x12},

	{SENSOR, 0x04, 0x02},
	{SENSOR, 0x05, 0xef, 0x00}
	/*end of sequence 6*/
};

#endif
