/* bayer.c
 *
 * Copyright  2001 Lutz Mller <lutz@users.sf.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details. 
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "bayer.h"

#define BLUE 0
#define GREEN 1
#define RED 2

#define AD(x, y, w) ((y)*(w)*3+3*(x))

void bayer_interpolate (unsigned char *image, int w, int h, BayerTile tile)
{
	int x, y, bayer;
	int p0, p1, p2, p3;
	int div, value;
	unsigned char *f[9];
	int W = 3 * w;
	f[0] = image - W - 3;
	f[1] = image - W;
	f[2] = image - W + 3;
	f[3] = image - 3;
	f[4] = image;
	f[5] = image + 3;
	f[6] = image + W - 3;
	f[7] = image + W;
	f[8] = image + W + 3;

	switch (tile) {
	default:
	case BAYER_TILE_RGGB:
	case BAYER_TILE_RGGB_INTERLACED:
		p0 = 3; p1 = 2; p2 = 1; p3 = 0;
		break;
	case BAYER_TILE_GRBG:
	case BAYER_TILE_GRBG_INTERLACED:
		p0 = 1; p1 = 3; p2 = 0; p3 = 2;
		break;
	case BAYER_TILE_BGGR:
	case BAYER_TILE_BGGR_INTERLACED:
		p0 = 0; p1 = 1; p2 = 2; p3 = 3;
		break;
	case BAYER_TILE_GBRG:
	case BAYER_TILE_GBRG_INTERLACED:	
		p0 = 2; p1 = 0; p2 = 3; p3 = 1;
		break;
	}

	for (y = 0; y < h; y++)
		for (x = 0; x < w; x++) {
			bayer = (x&1?0:1) + (y&1?0:2);
			if ( bayer == p0 ) {
				
				/* red. green lrtb, blue diagonals */
				div = value = 0;
				if (y) {
					value += *(f[1]+GREEN);
					div++;
				}
				if (y < (h - 1)) {
					value += *(f[7]+GREEN);
					div++;
				}
				if (x) {
					value += *(f[3]+GREEN);
					div++;
				}
				if (x < (w - 1)) {
					value += *(f[5]+GREEN);
					div++;
				}
				*(f[4]+GREEN) = value / div;

				div = value = 0;
				if ((y < (h - 1)) && (x < (w - 1))) {
					value += *(f[8]+BLUE);
					div++;
				}
				if ((y) && (x)) {
					value += *(f[0]+BLUE);
					div++;
				}
				if ((y < (h - 1)) && (x)) {
					value += *(f[6]+BLUE);
					div++;
				}
				if ((y) && (x < (w - 1))) {
					value += *(f[2]+BLUE);
					div++;
				}
				*(f[4]+BLUE) = value / div;

			} else if (bayer == p1) {
				
				/* green. red lr, blue tb */
				div = value = 0;
				if (x < (w - 1)) {
					value += *(f[5]+RED);
					div++;
				}
				if (x) {
					value += *(f[3]+RED);
					div++;
				}
				*(f[4]+RED) = value / div;

				div = value = 0;
				if (y < (h - 1)) {
					value += *(f[7]+BLUE);
					div++;
				}
				if (y) {
					value += *(f[1]+BLUE);
					div++;
				}
				*(f[4]+BLUE) = value / div;

			} else if ( bayer == p2 ) {
				
				/* green. blue lr, red tb */
				div = value = 0;
				
				if (x < (w - 1)) {
					value += *(f[5]+BLUE);
					div++;
				}
				if (x) {
					value += *(f[3]+BLUE);
					div++;
				}
				*(f[4]+BLUE) = value / div;

				div = value = 0;
				if (y < (h - 1)) {
					value += *(f[7]+RED);
					div++;
				}
				if (y) {
					value += *(f[1]+RED);
					div++;
				}
				*(f[4]+RED) = value / div;

			} else {
				
				/* blue. green lrtb, red diagonals */
				div = value = 0;

				if (y) {
					value += *(f[1]+GREEN);
					div++;
				}
				if (y < (h - 1)) {
					value += *(f[7]+GREEN);
					div++;
				}
				if (x) {
					value += *(f[3]+GREEN);
					div++;
				}
				if (x < (w - 1)) {
					value += *(f[5]+GREEN);
					div++;
				}
				*(f[4]+GREEN) = value / div;

				div = value = 0;
				if ((y < (h - 1)) && (x < (w - 1))) {
					value += *(f[8]+RED);
					div++;
				}
				if ((y) && (x)) {
					value += *(f[0]+RED);
					div++;
				}
				if ((y < (h - 1)) && (x)) {
					value += *(f[6]+RED);
					div++;
				}
				if ((y) && (x < (w - 1))) {
					value += *(f[2]+RED);
					div++;
				}
				*(f[4]+RED) = value / div;
			}
			f[0] += 3;
			f[1] += 3;
			f[2] += 3;
			f[3] += 3;
			f[4] += 3;
			f[5] += 3;
			f[6] += 3;
			f[7] += 3;
			f[8] += 3;
		}
}
